/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

/* convert:
   convert the global advances[] array to a basic techtree structure
*/   

#include <stdlib.h>

#include "tech.h"
#include "techtree.h"
#include "util.h"
#include "checkalloc.h"
#include "convert.h"

int
is_real_tech( /*enum tech_type_id*/ int i ) {
    return ( i!=A_NONE && i!=A_LAST &&
	     advances[i].req[0] != A_LAST &&
	     advances[i].req[0] != A_LAST );
}

/* The remap vector maps original tech indices to new indices,
   or -1 for "non-real" techs */
static void
init_remap( int *remap )
{
    int i, sum=0;

    for( i=0; i<A_LAST; i++ ) {
	if( is_real_tech(i) ) {
	    remap[i] = sum;
	    sum++;
	} else {
	    remap[i] = -1;
	}
    }
}

/* ENTRY FUNCTION: */   
void
advances_to_techtree( tech_tree_t *ttree )
{
    int *remap;			/* indexed by tech_type_id; new number */
    tech_node_t *node;
    int i, j, k_req;

    remap = malloc((A_LAST+1)*sizeof(int));
    CHECK_ALLOC(remap);
    init_remap( remap );

    techtree_init( ttree );
    
    for( i=0; i<A_LAST; i++ ) {
	if( remap[i] >= 0 ) {
	    node = techtree_addnode( ttree );
	    techtree_initnode( node );
	    node->orig_itech = i;
	    node->name = advances[i].name;
	    for( j=0; j<2; j++ ) {
		k_req = remap[advances[i].req[j]];
		if( k_req >=0 ) {
		    node->i_req[node->n_req] = k_req;
		    node->n_req++;
		}
	    }
	}
    }
    free( remap );
}

