/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "ilist.h"
#include "checkalloc.h"

#define ILIST_MAGIC 9235
#define ILIST_DO_MAGIC 1

#define ILIST_CHECK_MAGIC(il) \
    do { \
        if( ILIST_DO_MAGIC && il->magic != ILIST_MAGIC ) { \
	    fprintf( stderr, "bad ilist magic\n" ); \
	    exit(1); \
        } \
    } while(0)	    
					      

/* internal: */
static void
ilist_realloc( ilist_t *il, int n_new )
{
    ILIST_CHECK_MAGIC(il);
    il->i = realloc( il->i, n_new*sizeof(int) );
    CHECK_ALLOC(il->i);
    il->n_alloc = n_new;
}

/* external: */
void
ilist_new( ilist_t *il, int n  )
{
    il->magic = ILIST_MAGIC;
    il->n = 0;
    il->i = NULL;
    ilist_realloc( il, n );
}


void
ilist_renew( ilist_t *il, int n )
{
    ILIST_CHECK_MAGIC(il);
    il->n = 0;
    ilist_realloc( il, n );
}

void
ilist_free( ilist_t *il )
{
    ILIST_CHECK_MAGIC(il);
    il->n = 0;
    il->n_alloc = 0;
    free( il->i );
}

void
ilist_push( ilist_t *il, int v )
{
    ILIST_CHECK_MAGIC(il);
    if( il->n == il->n_alloc ) {
	ilist_realloc( il, 2*(il->n_alloc+1) );
    }
    il->i[il->n] = v;
    il->n++;
}

int
ilist_empty( ilist_t *il )
{
    ILIST_CHECK_MAGIC(il);
    return (il->n == 0);
}

int
ilist_endval( ilist_t *il )
{
    ILIST_CHECK_MAGIC(il);
    if( ilist_empty(il) ) {
	fprintf( stderr, "ilist_endval on empty list\n" );
	exit(1);
    }
    return il->i[il->n-1];
}

int
ilist_pop( ilist_t *il )
{
    int v;

    ILIST_CHECK_MAGIC(il);
    v = ilist_endval(il);	/* checks if there; get value */
    il->n--;			/* pop */
    return v;
}
