/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>

#include <shared.h>		/* mystrdup */

#include "options.h"

/* default options (if leave out, then default=zero) */
/* NOTE: integers!! */
/* Note: now some strings too... */
static void
default_options( techtree_opt_t *opt )
{
    opt->rulesetdir = "data/default";   /* ptr assign */
    
    opt->rm_redundant = 0;	/* ref:0 */
    
    opt->page_width = 210;	/* ref:210 (A4) */
    opt->page_height = 285;	/* ref:285, was:295 (A4), down a bit */
    opt->page_margin = 10;	/* ref:10 */

    opt->first_at_top = 1;	/* ref:1 */
    opt->tech_info = 1;		/* ref:1, was 0 */
    opt->two_page = 0;		/* ref:0 */
    opt->two_overlap = 1;	/* ref:1, was:5 */
    opt->line_gray = 50;	/* ref:50 */
    opt->line_width = 50;	/* ref:50 */

    opt->seed = 115332492;	/* ref1:15332492, was: 3129, was:9752 */
    opt->n_seed = 1;		/* ref:1 */

    opt->pseudo_size = 30;	/* ref:30 */
    opt->init_pseudo_linear = 1; /* ref:1 */
    opt->n_iter = 10;		/* ref:10, was:100 */
    opt->n_iter_swap = 5;	/* ref:5 */
    opt->n_iter_smooth = 5;	/* ref:5 */
    opt->n_iter_zero = 5;	/* ref:5 */
    opt->frep_n_inner = 1;	/* ref:1 */
    opt->frep_n_outer = 1;	/* ref:1 */
    opt->frep_size = 1;		/* ref:1 */
    opt->frep_do_pp = 1;	/* ref:1 */

    opt->debug = 0;		/* ref:0 */
    opt->debug_new = 0;		/* ref:0 */
    opt->debug_ps = 0;		/* ref:0 */
    opt->printps = 1;		/* ref:1 */
    opt->pseudo_labels = 0;	/* ref:0 */
}

struct istr_addr {
    int an_int;
    union {
	int *i;
	char **s;
    }u;
};

#define OP(x) #x ,
#define OP_STRING(x) #x ,
static char *option_names[] = {
#include "optionlist.h"
};
#undef OP
#undef OP_STRING

#define NUM_OPTIONS (sizeof(option_names)/sizeof(option_names[0]))

static struct istr_addr option_addrs[NUM_OPTIONS];
static struct option long_options[NUM_OPTIONS+1];


static void
init_option_addrs( techtree_opt_t *opt )
{
    int i = 0;
#define OP(x) \
    option_addrs[i].an_int = 1; \
    option_addrs[i].u.i = &(opt->x); \
    i++;
#define OP_STRING(x) \
    option_addrs[i].an_int = 0; \
    option_addrs[i].u.s = &(opt->x); \
    i++;
#include "optionlist.h"
#undef OP
#undef OP_STRING
}

static void
zero_out_options( techtree_opt_t *opt )
{
    int i;
    
    init_option_addrs(opt);
    for( i=0; i<NUM_OPTIONS; i++ ) {
	if( option_addrs[i].an_int ) {
	    *(option_addrs[i].u.i) = 0;
	} else {
	    *(option_addrs[i].u.s) = "";
	}
    }
}

static void
init_long_options( void ) {
    int i;

    for( i=0; i<NUM_OPTIONS; i++ ) {
	long_options[i].name = option_names[i];
	long_options[i].has_arg = 1;
	long_options[i].flag = 0;
	long_options[i].val = 0;
    }
    /* terminator: */
    i = NUM_OPTIONS;
    long_options[i].name = 0;
    long_options[i].has_arg = 0;
    long_options[i].flag = 0;
    long_options[i].val = 0;
}

/* requires init_option_addrs done: */
static void
print_usage( FILE *fp, char *argv0 ) {
    int i, c=0;
    
    fprintf( fp, "%s: Usage: %s [option=val] ...\n", argv0, argv0 );
    fprintf( fp, "Options:\n" );
    for( i=0; i<NUM_OPTIONS; i++ ) {
	if( option_addrs[i].an_int ) {
	    c += fprintf( fp, " -%s(=%d)", option_names[i],
			  *(option_addrs[i].u.i) );
	} else {
	    c += fprintf( fp, " -%s(=\"%s\")", option_names[i],
			  *(option_addrs[i].u.s) );
	}
	if( c >= 60 ) {
	    fprintf( fp, "\n" );
	    c = 0;
	}
    }
    if( c > 0 ) {
	fprintf( fp, "\n" );
    }
}
     
void
get_options( int *argc, char ***argv, techtree_opt_t *opt )
{
    zero_out_options( opt );
    default_options( opt );
    init_option_addrs(opt);
    init_long_options();
    for(;;) {
	int c, option_index;
	
	c = getopt_long_only( *argc, *argv, "", long_options, &option_index );
	if( c == EOF ) {
	    break;
	}
	if( c!=0 ) {
	    /* fprintf( stderr, "Unexpected return from getopt (%d)\n", c ); */
	    print_usage( stderr, (*argv)[0] );
	    exit(1);
	}
	if( option_addrs[option_index].an_int ) {
	    *(option_addrs[option_index].u.i) = atoi(optarg);
	} else {
	    *(option_addrs[option_index].u.s) = mystrdup(optarg);
	}
		   
    }
}

void
print_options( FILE *fp, techtree_opt_t *opt, char *version, char *prefix )
{
    int i;
    
    fprintf( fp, "%stechtree v%s options:\n", prefix, version );
    init_option_addrs(opt);
    for( i=0; i<NUM_OPTIONS; i++ ) {
	if( option_addrs[i].an_int ) {
	    fprintf( fp, "%s %s = %d\n", prefix, option_names[i],
		     *(option_addrs[i].u.i) );
	} else {
	    fprintf( fp, "%s %s = \"%s\"\n", prefix, option_names[i],
		     *(option_addrs[i].u.s) );
	}
    }
}
