/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdio.h>

#include "techtree.h"
#include "print.h"

void
print_full_node( FILE *fp, tech_tree_t *ttree, int i )
{
    tech_node_t *node = TNODE_P(ttree,i);
    int j;
    
    fprintf( fp, "i=%d i_sort=%d name ===== \"%s\"\n", i,
	    (ttree->i_sort?ttree->i_sort[i]:-1), node->name );
    fprintf( fp, "orig_itech=%d pseudo=%d\n", node->orig_itech, node->pseudo );
    fprintf( fp, "n_req=%d", node->n_req );
    for( j=0; j<node->n_req; j++ ) {
	fprintf( fp, " %d\"%s\"%s", node->i_req[j],
		 ttree->nodes[node->i_req[j]].name,
		 node->redundant_req[j]?"*redundant*":"" );
    }
    fprintf( fp, "\n" );
    fprintf( fp, "n_allow=%d", node->n_allow );
    for( j=0; j<node->n_allow; j++ ) {
	fprintf( fp, " %d\"%s\"%s", node->i_allow[j],
		 ttree->nodes[node->i_allow[j]].name,
		 node->redundant_allow[j]?"*redundant*":"" );
    }
    fprintf( fp, "\n" );
    fprintf( fp, "n_level_req=%d", node->n_level_req );
    for( j=0; j<node->n_level_req; j++ ) {
	fprintf( fp, " %d", node->level_req[j] );
    }
    fprintf( fp, "\n" );
    fprintf( fp, "level=%d, dy=%g, dx=%g, dx_prev=%g\n", 
	    node->level, node->dy, node->dx, node->dx_prev );
}

void
print_full_tree( FILE *fp, tech_tree_t *ttree )
{
    int i;

    fprintf( fp, "TREE: n %d n_alloc %d n_levels %d\n",
	    ttree->n, ttree->n_alloc, ttree->n_levels );
    fprintf( fp, "pointers nodes %p per_level %p real_per_level %p\n"
	    " i_sort %p level_start %p\n",
	    ttree->nodes, ttree->per_level, ttree->real_per_level,
	    ttree->i_sort, ttree->level_start );
    if( ttree->n_levels > 0 ) {
	fprintf( fp, "ilevel, per_level, real_per_level, level_start:\n" );
	for( i=0; i<ttree->n_levels; i++ ) {
	    fprintf( fp, "%d %d %d %d\n", i,
		    (ttree->per_level?ttree->per_level[i]:-1),
		    (ttree->real_per_level?ttree->real_per_level[i]:-1),
		    (ttree->level_start?ttree->level_start[i]:-1) );
	}
    }
    printf( "max_one_level=%d, pseudo_size=%g, neff_max=%g, one_wide=%g\n",
	    ttree->max_one_level, ttree->pseudo_size, ttree->neff_max,
	    ttree->one_wide );
    for( i=0; i<ttree->n; i++ ) {
	if( i!=0 ) {
	    fprintf( fp, "\n" );
	}
	print_full_node( fp, ttree, i );
    }
}

void
print_tree_sort_dx( FILE *fp, tech_tree_t *ttree )
{
    tech_node_t *node;
    int i, j;

    for( i=0; i<ttree->n; i++ ) {
	j = ttree->i_sort[i];
	node = ttree->nodes + j;
	fprintf( fp, "%d %d %g \"%s\"\n",
		 i, node->level, node->dx, node->name );
    }
}
