/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdio.h>

#include "techtree.h"
#include "redundant.h"
#include "tech.h"		/* advances[] */
#include "convert.h"		/* is_real_tech() */

static int
check_redundant( int tech, int goal )
{
    int j, req;;
    
    if( !is_real_tech(tech) ) {
	return 0;
    }
    if( tech == goal ) {
	return 1;
    }
    for( j=0; j<2; j++ ) {
	req = advances[tech].req[j];
	if( check_redundant( req, goal ) ) {
	    return 1;
	}
    }
    return 0;
}

void
rm_redundant( tech_tree_t *ttree, int print, FILE *where )
{
    int i, j, to_check, res;
    
    for( i=0; i<A_LAST; i++ ) {
	for( j=0; j<2; j++ ) {
	    to_check = advances[i].req[j];
	    if( is_real_tech(to_check) ) {
		res = check_redundant(advances[i].req[1-j], to_check);
		if( res ) {
		    if( print ) {
			fprintf( where, "redundant req: tech %s req %s\n",
				advances[i].name,
				advances[to_check].name );
		    }
		    advances[i].req[j] = A_NONE;
		}
	    }
	}
    }
}

static int
check_redundant_tree( tech_tree_t *ttree, int i_head, int i_node, int i_goal )
{
    tech_node_t *node;
    int j, i_next;
    
    if( i_node == i_goal ) {
	return 1;
    }
    node = TNODE_P(ttree,i_node);
    for( j=0; j<node->n_req; j++ ) {
	i_next = node->i_req[j];
	/* don't check the direct link: */
	if( !(i_node == i_head && i_next == i_goal) ) {
	    if( check_redundant_tree( ttree, i_head, i_next, i_goal ) ) {
		return 1;
	    }
	}
    }
    return 0;
}

void
mark_redundant( tech_tree_t *ttree )
{
    tech_node_t *node, *lnode;
    int i, j, k, to_check, res;

    for( i=0, node=ttree->nodes; i<ttree->n; i++, node++ ) {
	for( j=0; j<node->n_req; j++ ) {
	    to_check = node->i_req[j];
	    res = check_redundant_tree( ttree, i, i, to_check );
	    if( res ) {
		node->redundant_req[j] = 1;
		lnode = TNODE_P(ttree, to_check);
		for( k=0; k<lnode->n_allow; k++ ) {
		    if( lnode->i_allow[k] == i ) {
			lnode->redundant_allow[k] = 1;
			break;
		    }
		}
	    }
	}
    }
}

