/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdlib.h>

#include "techtree.h"
#include "sort.h"
#include "checkalloc.h"

static tech_tree_t *stree;

/* for qsort; inputs are indices to stree->nodes */
int
tcompare_levels( const void* i1, const void* i2 )
{
    return stree->nodes[*(int*)i1].level 
	- stree->nodes[*(int*)i2].level; 
}

int
tcompare_lev_dx( const void* i1, const void* i2 )
{
    int idiff;
    float fdiff;

    idiff = stree->nodes[*(int*)i1].level 
	- stree->nodes[*(int*)i2].level;
    if( idiff ) {
	return idiff;
    } else {
	fdiff = stree->nodes[*(int*)i1].dx 
	    - stree->nodes[*(int*)i2].dx;
	return (fdiff<0 ? -1 : fdiff>0 ? 1 : 0 );
    }
}

int
tcompare_lev_i( const void* i1, const void* i2 )
{
    int idiff;
    
    idiff = stree->nodes[*(int*)i1].level 
	- stree->nodes[*(int*)i2].level;
    if( idiff ) {
	return idiff;
    }
    return stree->nodes[*(int*)i1].orig_itech 
	- stree->nodes[*(int*)i2].orig_itech;
}

int
tcompare_dx( const void* i1, const void* i2 )
{
    float fdiff;

    fdiff = stree->nodes[*(int*)i1].dx 
	- stree->nodes[*(int*)i2].dx;
    return (fdiff<0 ? -1 : fdiff>0 ? 1 : 0 );
}

void
sort_techtree_part( tech_tree_t *ttree, int *i_list, int n,
		    qsort_arg_fn_t tcompare )
{
    stree = ttree;
    qsort( i_list, n, sizeof(int), tcompare );
}

void
sort_techtree( tech_tree_t *ttree, qsort_arg_fn_t tcompare )
{
    int j;

    stree = ttree;
    stree->i_sort = realloc( stree->i_sort, stree->n*sizeof(int) );
    CHECK_ALLOC(stree->i_sort);
    for( j=0; j<stree->n; j++ ) {
	stree->i_sort[j] = j;
    }
    qsort( stree->i_sort, stree->n, sizeof(int), tcompare );
}



