/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#ifndef TT__TECHTREE_H
#define TT__TECHTREE_H

/* Max links (dependency relationships) from given tech in either forward
   or backwards direction; actually backwards links are always
   either 0 or 1 or 2, but we generalize...
*/
#define MAX_LINKS 10
#define NDIM 2

typedef struct {
    char *name;			/* tech name; ptr to name in advances[] */
    int orig_itech;		/* index of position in advances[] */
    int pseudo;			/* pseudo nodes are place-fillers */
    int i_req[MAX_LINKS];	/* indices to tree->nodes */
    int n_req;			/* req = "... is REQuirement for this tech" */
    int i_allow[MAX_LINKS];	/* indices to tree->nodes */
    int n_allow;		/* allow = "this tech ALLOWS ..." */
    int level_req[MAX_LINKS];	/* 1+req[level] for each req (different order) */
    int n_level_req;		/* how many of level_req done so far */
    int redundant_req[MAX_LINKS]; /* 0 or 1 for each req */
    int redundant_allow[MAX_LINKS]; /* 0 or 1 for each allow */
    int level;			/* max(level_req); height in tree */
    float dy;			/* level, normalized to [0,1] */
    float dx;			/* horizontal position in tree, in [0,1] */
    float dx_prev;		/* dx on previous iteration */
    float x[NDIM];		/* position in new_layout */
    float force[NDIM];		/* "force" in new_layout */
    float force_prev[NDIM];	/* force in previous iteration */
} tech_node_t;

typedef struct {
    tech_node_t *nodes;		/* array of nodes */
    int n;			/* number of nodes */
    int n_alloc;		/* number allocated */
    int n_levels;		/* number of levels */
    int *per_level;		/* number of nodes at each level */
    int *real_per_level;	/* not counting pseudo nodes */
    int *i_sort;		/* indices to nodes with some sorting,
				   typically by level */
    int *level_start;		/* index to i_sort for start of
				   each level */
    int max_one_level;		/* max number of nodes in one level */
    float pseudo_size;		/* relative "width" of pseudo nodes */
    float neff_max;		/* max sum(sizes) in one level */
    float one_wide;		/* width of one normal node */
    float metric_dist;
    int metric_cross;
    float metric_distx;
} tech_tree_t;

/* node index to node pointer: */
#define TNODE_P(t,i) ((t)->nodes+(i))

/* node pointer to node index: */
#define TNODE_I(t,n) ((n)-(t)->nodes)

/* node relative width (after pseudo_size setup) */
#define TNODE_REL_WIDTH(t,n) ((n)->pseudo?(t)->pseudo_size:1.0)

/* node width (wrt [0,1]) (after pseudo_size,one_wide setup) */
#define TNODE_WIDTH(t,n) ((t)->one_wide*TNODE_REL_WIDTH((t),(n)))


extern char *version;

#endif /* TT__TECHTREE_H */
