/********************************************************************** 
 techtree - Copyright (C) 1998 - David W. Pfitzner
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#include <stdlib.h>

#include "techtree.h"
#include "util.h"
#include "checkalloc.h"

/* initializes an empty tree */
void
techtree_init( tech_tree_t *ttree )
{
    ttree->nodes = NULL;
    ttree->n = ttree->n_alloc = ttree->n_levels = 0;
    ttree->per_level = NULL;
    ttree->real_per_level = NULL;
    ttree->i_sort = NULL;
    ttree->level_start = NULL;
    ttree->max_one_level = 0;
    ttree->pseudo_size = ttree->neff_max = ttree->one_wide = 0.0;
    ttree->metric_dist = ttree->metric_distx = 0.0;
    ttree->metric_cross = 0;
}

/* returns the index of the new node */
tech_node_t*
techtree_addnode( tech_tree_t *ttree )
{
    int n_new;
    
    if( ttree->n == ttree->n_alloc ) {
	n_new = 2*(ttree->n_alloc+1);
	ttree->nodes = realloc( ttree->nodes, n_new*sizeof(tech_node_t));
	CHECK_ALLOC(ttree->nodes);
	ttree->n_alloc = n_new;
    }
    ttree->n++;
    return TNODE_P(ttree,ttree->n-1);
}

/* initializes a "blank" node */
void
techtree_initnode( tech_node_t *node )
{
    int i;
    
    node->name = "";		/* pointer to static emtpy string */
    node->orig_itech = -1;
    node->pseudo = 0;
    node->n_req = 0;
    node->n_allow = 0;
    node->n_level_req = 0;
    node->level = 0;
    node->dy = 0.0;
    node->dx = 0.0;
    node->dx_prev = 0.0;

    for( i=0; i<MAX_LINKS; i++ ) {
	node->i_allow[i] = node->i_req[i] = node->level_req[i]
	    = node->redundant_req[i] = node->redundant_allow[i] = 0;
    }
    for( i=0; i<NDIM; i++ ) {
	node->x[i] = node->force[i] = node->force_prev[i] = 0.0;
    }
}


